// SRP Statistics Header
// Defines constructs needed to perform srp network statistics

// Created 11/21/99
// Modified 11/21/99 

// 11/21  mpiecuch       initial creation

#include <sys/time.h>
#include <unistd.h>

struct srp_stats {
  long int data_bytes_sent;
  long int retrans_bytes_sent;
  long int data_pkts_sent;
  long int retrans_sent;
  long int buffer_misses;
  long int data_pkts_recv;
  long int data_bytes_recv;
  long int retrans_rqstd;
  long int retrans_bytes_rqstd;
  long int give_ups;
  long int too_old;
  long int too_new;
  long int cache_hits;

};

struct srp_stats stat;

// Server input variables
int SRP_BUFFER_SIZE = 10;
int SRP_MAX_DATA_ID = 100;
int NUM_FRAMES = 10000;
int FRAME_SPEED = 1000000;
int FRAME_SIZE = 0;

// Client input variables
int SRP_NUM_RETRIES = 5;
int SRP_RTO = 100000;


void srp_stat_print(struct srp_stats stat) {
  printf("Data Packets Sent: %ld (%ld bytes)\n", stat.data_pkts_sent, stat.data_bytes_sent);
  printf("Retransmitions Sent: %ld (%ld bytes)\n", stat.retrans_sent, stat.retrans_bytes_sent);
  printf("Buffer Misses: %ld\n", stat.buffer_misses);

}

void srp_stat_client_print(struct srp_stats stat) {
  printf("Data Packets Recv: %ld (%ld bytes)\n", stat.data_pkts_recv, stat.data_bytes_recv);
  printf("Retransmitions Requested: %ld (%ld bytes)\n", stat.retrans_rqstd, stat.retrans_bytes_rqstd);
  printf("Give up=%ld\n", stat.give_ups);
  printf("Too Old=%ld  Too New=%ld\n", stat.too_old, stat.too_new);
  printf("Cache Hits=%ld\n", stat.cache_hits);
}

void srp_stat_server_write(struct srp_stats stat, char *filename) {
  FILE *outfile;

  if ((outfile = fopen(filename, "wt")) == NULL) {
    printf("Error, could not open statistics file \"%s\" for writing\n", filename);
    exit(1);
  }

  fprintf(outfile, "DATA_PACKETS_SENT=%ld\n", stat.data_pkts_sent);
  fprintf(outfile, "DATA_PACKETS_SENT_BYTES=%ld\n", stat.data_bytes_sent);
  fprintf(outfile, "RETRANS_PACKETS_SENT=%ld\n", stat.retrans_sent);
  fprintf(outfile, "RETRANS_PACKETS_SENT_BYTES=%ld\n", stat.retrans_bytes_sent);
  fprintf(outfile, "BUFFER_MISSES=%ld\n", stat.buffer_misses);

  fclose(outfile);

}

void srp_stat_client_write(struct srp_stats stat, char *filename) {
  FILE *outfile;

  if ((outfile = fopen(filename, "wt")) == NULL) {
    printf("Error, could not open statistics file \"%s\" for writing\n", filename);
    exit(1);
  }

  fprintf(outfile, "DATA_PACKETS_RECV=%ld\n", stat.data_pkts_recv);
  fprintf(outfile, "DATA_PACKETS_RECV_BYTES=%ld\n", stat.data_bytes_recv);
  fprintf(outfile, "RETRANS_PACKETS_RQSTD=%ld\n", stat.retrans_rqstd);
  fprintf(outfile, "RETRANS_PACKETS_RQSTD_BYTES=%ld\n", stat.retrans_bytes_rqstd);
  fprintf(outfile, "GIVE_UPS=%ld\n", stat.give_ups);
  fprintf(outfile, "TOO_OLD=%ld\n", stat.too_old);
  fprintf(outfile, "TOO_NEW=%ld\n", stat.too_new);
  fprintf(outfile, "CACHE_HITS=%ld\n", stat.cache_hits);

  fclose(outfile);

}

void srp_stat_server_read(char *filename) {
  FILE *infile;
  char buffer[255], strvar[50], strval[50];
  char *tmp;
  long int val;


  if ((infile = fopen(filename, "rt")) == NULL) {
    printf("Error, could not open statistics file \"%s\"for reading\n", filename);
  }

  while(!feof(infile)) {
    fgets(buffer, 255, infile);

      strcpy(strvar,(char *) strtok(buffer, "="));

      tmp = strtok(NULL, "=");
      if  (tmp != NULL) {
	strcpy(strval, tmp);

 
	val = atoi(strval);
      
	if (strcmp(strvar, "SRP_BUFFER_SIZE") == 0) {
	  printf("%s=%ld\n", strvar, val);
	  SRP_BUFFER_SIZE = val;
	}
      
	if (strcmp(strvar, "SRP_MAX_DATA_ID") == 0) {
	  printf("%s=%ld\n", strvar, val);
	  SRP_MAX_DATA_ID = val;
	}
      
	if (strcmp(strvar, "NUM_FRAMES") == 0) {
	  printf("%s=%ld\n", strvar, val);
	  NUM_FRAMES = val;
	}
      
	if (strcmp(strvar, "FRAME_SPEED") == 0) {
	  printf("%s=%ld\n", strvar, val);
	  FRAME_SPEED = val;
	}
      
	if (strcmp(strvar, "FRAME_SIZE") == 0) {
	  printf("%s=%ld\n", strvar, val);
	  FRAME_SIZE = val;
	}
      }

  }
  
}


void srp_stat_client_read(char *filename) {
  FILE *infile;
  char buffer[255], strvar[50], strval[50];
  char *tmp;
  long int val;


  if ((infile = fopen(filename, "rt")) == NULL) {
    printf("Error, could not open statistics file \"%s\"for reading\n", filename);
  }

  while(!feof(infile)) {
    fgets(buffer, 255, infile);

    strcpy(strvar,(char *) strtok(buffer, "="));

    tmp = strtok(NULL, "=");
    if (tmp != NULL) {
      strcpy(strval,(char *) tmp);
 
      val = atoi(strval);
      
      if (strcmp(strvar, "SRP_BUFFER_SIZE") == 0) {
	printf("%s=%ld\n", strvar, val);
	SRP_BUFFER_SIZE = val;
      }
      
      if (strcmp(strvar, "SRP_MAX_DATA_ID") == 0) {
	printf("%s=%ld\n", strvar, val);
	SRP_MAX_DATA_ID = val;
      }
      
      if (strcmp(strvar, "FRAME_SPEED") == 0) {
	printf("%s=%ld\n", strvar, val);
	FRAME_SPEED = val;
      }
      
      if (strcmp(strvar, "SRP_NUM_RETRIES") == 0) {
	printf("%s=%ld\n", strvar, val);
	SRP_NUM_RETRIES = val;
      }
      
      if (strcmp(strvar, "SRP_RTO") == 0) {
	printf("%s=%ld\n", strvar, val);
	SRP_RTO = val;
      }

    }

  }

}



void srp_stat_init(struct srp_stats *stat) {

  stat->data_bytes_sent=0;
  stat->retrans_bytes_sent=0;
  stat->data_pkts_sent=0;
  stat->retrans_sent=0;
  stat->buffer_misses=0;
  stat->data_pkts_recv=0;
  stat->data_bytes_recv=0;
  stat->retrans_rqstd=0;
  stat->retrans_bytes_rqstd=0;
  stat->give_ups=0;
  stat->too_old=0;
  stat->too_new=0;
  stat->cache_hits=0;  

}

void timestamp(char *time_file) {
  struct timeval cur_time;
  struct timezone dummy;
  FILE *outfile;

  if ((outfile = fopen(time_file, "at")) == NULL) {
    printf("Error, could not open statistics file \"%s\" for writing\n", time_file);
    exit(1);
  }


  if (gettimeofday(&cur_time, &dummy) >= 0) {
    fprintf(outfile, "%ld,%ld\n", cur_time.tv_sec, cur_time.tv_usec);
  } else {
    fprintf(outfile, "Error\n");
  }
  fclose(outfile);

}











