// SRP Server Application
// srpserverapp.c
// Created: 12/7/99
// Modified: 12/7/99

// 12-7  mpiecuch     Initial Creation 

// Compile: gcc -lpthread srpserverapp.c


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "srpserverlib.c"


// Main defined to test srp code
int main(int argc, char **argv) {

  char test_data[SRP_MAX_DATA_SIZE];
  int test_int, i;
  struct srp_server_socket *srp_sock;
  int udp_sock;
  struct sockaddr_in localaddr, peeraddr;
  int size_localaddr, port;
  char host[50];
  struct srp_nack_packet srp_ready_msg;
  struct timeval tv;
  FILE *testdatafile;
  int testdatasize;


#ifdef SRP_STAT
  printf("Statistics Version\n");
  srp_stat_init(&stat);
  srp_stat_server_read("server_params.txt");
#endif

  if (argc == 2) {
    testdatafile = fopen(argv[1], "r");
  }

  tv.tv_sec = 0;
  tv.tv_usec = FRAME_SPEED * 1000;


  // Create normal UDP socket
  if ((udp_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {perror("Socket Error! "); return(EINVAL);}

  // Setup necessary UDP socket stuff
  localaddr.sin_family = AF_INET;
  localaddr.sin_port = htons(SRP_PORT);
  localaddr.sin_addr.s_addr = INADDR_ANY;
  size_localaddr = sizeof(localaddr);


  // Bind socket and get host/port info
  if ((bind(udp_sock, (struct sockaddr *) &localaddr, sizeof(localaddr))) < 0) {perror("Bind Error! "); exit(1);}
  if ((getsockname(udp_sock, (struct sockaddr *)  &localaddr, &size_localaddr )) < 0) {perror("Get Port Error! "); exit(1);}
  port = localaddr.sin_port;
  if ((gethostname(host, sizeof(host))) < 0) {perror("Get Host Error! "); exit(1);}

  printf("host: %s, port: %d\n", host, ntohs(port));


  // Get a ready (nack right now) message from the client to get the address to send to.
  recvfrom(udp_sock, &srp_ready_msg, sizeof(struct srp_nack_packet), 0, (struct sockaddr *) &peeraddr, &size_localaddr);

  printf("Initializing srp with: %s\n", inet_ntoa(peeraddr.sin_addr));


  // Create SRP socket and start retransmission listening thread
  srp_sock = srp_server_start(udp_sock, 0, (struct sockaddr *) &peeraddr, sizeof(struct sockaddr));



  // Keep sending test data to client every 5 seconds
  test_int = 0;

  if (FRAME_SIZE == 0) {
    testdatasize = random() % SRP_MAX_DATA_SIZE;
  } else {
    testdatasize = FRAME_SIZE;
  }

  while(test_int<=NUM_FRAMES) {
    // printf("Sending frame: %d\n", test_int);

    if (argc != 2) {
      
      for (i=0;i<testdatasize;i++) {
	test_data[i] = (char) ((random() % ('Z'-'A')) + 'A');
      }
      // Note: \0 is to make sure test client can print data as string
      srp_sendto(srp_sock, test_data, strlen(test_data));

    } else {

      fread(&test_data, testdatasize, 1, testdatafile);
      if (feof(testdatafile)) {
	printf("End of testdata file\n");
	fclose(testdatafile);
	srp_server_stop(srp_sock);
	
#ifdef SRP_STAT
	srp_stat_print(stat);
	srp_stat_server_write(stat, "server_stats.txt");
#endif
	
	exit(0);
      }
      srp_sendto(srp_sock, test_data, testdatasize);
      //srp_print_buffer(srp_sock);
    }

#ifdef SRP_STAT
    srp_stat_server_write(stat, "server_stats.txt");
#endif

    select(0, NULL, NULL, NULL, &tv);
    tv.tv_sec = 0;
    tv.tv_usec = FRAME_SPEED * 1000;
    test_int++;
  }
  
  srp_server_stop(srp_sock);

  if (argc == 2) {
    fclose(testdatafile);
  }

#ifdef SRP_STAT
  srp_stat_print(stat);
  srp_stat_server_write(stat, "server_stats.txt");
#endif


}
