// Implements a client
import java.io.*;
import java.net.*;
import java.util.*;

public class Client {
 
  static final int UDP = 0;
  static final int TCP = 1;
  static final int SRP = 2;

  public static void main(String[] args) {
    Client c = new Client();
    c.run(args);
  }  

  public void run(String[] args) {
    StringTokenizer st; 
    int protocol = 0;
    
    String outputfile = null;
    String inputfile = null;
    String host = null;
    int port = 0;
    int sleep = 100;

    if (args.length == 1)
      inputfile = args[0];
    else {
      System.out.println("Usage: java client (parameterfile)");
      System.exit(1);
    }
    try {
      String input = null, tok = null, val = null;
      BufferedReader in = new BufferedReader(new FileReader(inputfile));
      while((input = in.readLine()) != null) {
	st = new StringTokenizer(input, "=");
	tok = st.nextToken();
	val = st.nextToken();
	System.out.println(tok + ":" + val);
	if (tok != null){
	  if(tok.equals("protocol")){
	    if (val.equals("udp"))
	      protocol = UDP;
	    else if (val.equals("tcp"))
	      protocol = TCP;
	    else if (val.equals("srp"))
	      protocol = SRP;
	  }
	  if (tok.equals("port")){
	    port = Integer.parseInt(val);
	  }
	  if (tok.equals("outputfile")){
	    outputfile = val;
	  }
	  if (tok.equals("host")){
	    host = val;
	  }
	  if (tok.equals("sleep")){
	    sleep = Integer.parseInt(val);
	  }
	}
      } 
    } catch(NumberFormatException e){
      System.out.println("Improper port number");
      System.exit(1);
    } catch (IOException e) {
      System.out.println("Unable to read input file " + inputfile);
      System.exit(1);
    }
    
    if (outputfile == null){
      System.out.println("Output file not specified as a parameter");
      System.exit(1);
    }
    switch (protocol) {
    case UDP:
      udp(host, port, outputfile, sleep);
      break;
    case TCP:
      tcp(host, port, outputfile, sleep);
      break;
    case SRP:
      break;
    default:
      System.out.println("unknown protocol");
      break;
    }

  }


  private static void udp(String host, int port, String outputfile, int sleep){

    DatagramSocket socket = null;
    try{
      // get a datagram socket
      socket = new DatagramSocket();
      //      socket.setSoTimeout(10000);
      
      System.out.println("Started UDP client. Press ENTER to exit");
      InetAddress address = InetAddress.getByName(host);
      
      ClientBuffer cBuffer = new ClientBuffer();
      
      ClientUDPSendThread sender = null; 
      ClientUDPReceiveThread receiver = null;
	
      try{
	sender = 
	  new ClientUDPSendThread(socket, address, port, 
				  cBuffer, outputfile, sleep);
	receiver = 
	  new ClientUDPReceiveThread(socket, address, port, cBuffer, outputfile);
      } catch (IOException e) {
	System.out.println("Unable to create output files");
	System.exit(1);
      }
      
      sender.start();
      receiver.start();
	
      BufferedReader in = 
	new BufferedReader(new InputStreamReader(System.in));
      String input = in.readLine();
	
      sender.output();
      receiver.output();
      sender.stop();
      receiver.stop();
      
    } catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
    socket.close();
    System.exit(0);
  }
  

  private static void tcp(String host, int port, String outputfile, int sleep) {

    Socket socket = null;
    try{
      // get a datagram socket
//      host = "melbourne.srp1.net";
      InetAddress address = InetAddress.getByName(host);
      System.out.println(host + ":" + address + ":" + port);
      socket = new Socket(address, port);
      //      socket.setSoTimeout(10000);
      
      System.out.println("Started TCP client. Press ENTER to exit");
      
      ClientBuffer cBuffer = new ClientBuffer();
      
      ClientTCPSendThread sender = null; 
      ClientTCPReceiveThread receiver = null;
	
      try{
	sender = 
	  new ClientTCPSendThread(socket, address, port, 
				  cBuffer, outputfile, sleep);
	receiver = 
	  new ClientTCPReceiveThread(socket, address, port, cBuffer, outputfile);
      } catch (IOException e) {
	System.out.println("Unable to create output files");
	System.exit(1);
      }
      
      receiver.start();
      sender.start();
      
      BufferedReader in = 
	new BufferedReader(new InputStreamReader(System.in));
      String input = in.readLine();
      
//      sender.suspend();
//      sender.output();
      sender.stop();
      sender.output();
      Thread.currentThread().sleep(1000);
      receiver.output();
      receiver.stop();
      
    } catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
    try{
      socket.close();
    }catch(IOException e){
      System.out.println("Unable to close socket: " + e.getMessage());
    }
    System.exit(0);
  }
/*
  public void sleep(long time){
    try{
      wait(time);
    }catch (Exception e){
      System.out.println("ending pause");
    }
  }
*/
}


