// Implements a local server
import java.net.*;
import java.io.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;


public class Server {

  ServerSocket sSocket = null;           // Server socket
  DatagramSocket dSocket = null;           // Server socket
  Socket cSocket;                        // Socket for client
  int noClients = 0;                          // No client period count
  static int portNum = 1999;       // port number to run server on

  static final int UDP = 0;
  static final int TCP = 1;
  static final int SRP = 2;
  
  public static void main(String[] args) {
    StringTokenizer st; 
    int protocol = 0;

    String outputfile = null;
    String inputfile = null;

    if (args.length == 1)
      inputfile = args[0];
    else {
      System.out.println("Usage: java server (parameterfile)");
      System.exit(1);
    }
    try {
      String input = null, tok = null, val = null;
      BufferedReader in = new BufferedReader(new FileReader(inputfile));
      while((input = in.readLine()) != null) {
	st = new StringTokenizer(input, "=");
	tok = st.nextToken();
	val = st.nextToken();
	System.out.println(tok + ":" + val);
	if (tok != null){
	  if(tok.equals("protocol")){
	    if (val.equals("udp"))
	      protocol = UDP;
	    else if (val.equals("tcp"))
	      protocol = TCP;
	    else if (val.equals("srp"))
	      protocol = SRP;
	  }
	  if (tok.equals("port")){
	    portNum = Integer.parseInt(val);
	  }
	  if (tok.equals("outputfile")){
	    outputfile = val;
	  }
	}
      } 
    } catch(NumberFormatException e){
      System.out.println("Improper port number");
      System.exit(1);
    } catch (IOException e) {
      System.out.println("Unable to read input file " + inputfile);
      System.exit(1);
    }

    if (outputfile == null){
      System.out.println("Output file not specified as a parameter");
      System.exit(1);
    }
    switch (protocol) {
    case UDP: 
      udp(outputfile);
      break;
    case TCP:
      tcp(outputfile);
      break;
    case SRP:
      break;
    default:
      System.out.println("unknown protocol");
      break;
    }


  }

  public static void udp(String filename) {
    try {
      UDPThread udp = new UDPThread(portNum, filename);
      udp.start();
      System.out.println("Started UDP thread, press ENTER to exit");
      BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
      String input = in.readLine();
      udp.output();
      udp.stop();
      System.exit(0);
    }catch(IOException e) {
      System.out.println("UDP ERROR IN SERVER: " + e);
    } 
  }
  
  public static void tcp(String filename) {
    try {
      TCPThread tcp = new TCPThread(portNum, filename);
      tcp.start();
      System.out.println("Started TCP thread, press ENTER to exit");
      BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
      String input = in.readLine();
      tcp.output();
      tcp.stop();
      System.exit(0);
    }catch(IOException e) {
      System.out.println("TCP ERROR IN SERVER: " + e);
    } 
  }
  

//    public void keyTyped(KeyEvent e) { }
//    public void keyPressed(KeyEvent e) { }
//    public void keyReleased(KeyEvent e) { }
//    class KeyHandler extends KeyAdapter {
//      // Handle the key pressed event
//      public void keyPressed(KeyEvent e) {
//        System.out.println("HELLO");
//        if(e.getModifiers()==Event.CTRL_MASK) {
//          int code = e.getKeyCode();
//          if( code == KeyEvent.VK_C){
//             System.out.println("hello! -- [CTRL-C]");
//  	   System.exit(1);
//  	}
//        }
//      }
//    }
}
